require 'spec_helper'

describe Rents::Transaction do
  before(:all) do
    @protocol = 'http://'
    @domain = 'localhost:7000'
    @base_url = "#{@protocol}#{@domain}"
    @global_subscription_rid = get_json("#{@base_url}/api/v1/global_subscription")[:rid]
  end

  context 'BuyOperatorPage' do
    # OK Operator Page tests
    describe '(SUCCESS REQUEST)' do
      context 'amount with cents' do
        # ================    SetUp/Config    ================
        before(:all) do
          Rents.debug=true
          Rents.test_env=true

          # "Int" amount, with cents
          amount = (BigDecimal Random.rand(99999))/100
          card_brand = 'elo'

          # (* optional) used only for CieloPage
          redir_link = "#{@base_url}/api/redirect_receiver"

          # With cents
          aux = page_transaction_mock(card_brand, amount, redir_link)
          @page_resp = aux[:resp]
          @page_transaction = aux[:transaction]
        end

        # ================    Tests/Expects/Should   ================
        it 'resp should not be null' do
          expect(@page_resp).to_not be_nil
        end

        it 'resp should contain the RequestID' do
          request_id = @page_resp[:request_id]
          expect(request_id).to_not be_nil
          expect(request_id).to be_a Integer
        end

        it 'should setup the rid accessible method' do
          expect(@page_transaction.rid).to_not be_nil
        end

        it 'should setup the purchase_url accessible method' do
          expect(@page_transaction.purchase_url).to_not be_nil
        end

        it 'resp should not have any error' do
          error = @page_resp[:error]
          error = @page_resp['error'] if error.nil?
          expect(error).to be_nil
        end

        it 'resp should be an accessible Operator URL' do
          url = @page_transaction.purchase_url

          expect(url).to_not be_nil
          expect(accessible?(url)).to be_truthy
        end

        it 'must be verifiable' do
          # SetUps
          verify_resp = @page_transaction.verify
          error = verify_resp[:error]
          error = verify_resp['error'] if error.nil?

          # Validations
          expect(error).to be_nil
          expect(verify_resp).to_not be_nil

          # It is just created status
          expect(0..1).to cover(verify_resp[:status][:code])
          expect(verify_resp[:status][:name]).to eq('pending')
        end

        it 'must have a get accessible URL request' do
          url = @page_transaction.url_requested+@page_transaction.request_params.it_keys_to_get_param
          expect(be_accessible(url)).to be_truthy
        end

        it 'amount must be the same as send' do
          expect(@page_transaction.amount.to_i).to be == @page_resp[:transaction][:amount]
        end
      end

      context 'full amount (without any cent)' do
        # ================    SetUp/Config    ================
        before(:all) do
          Rents.debug=true
          Rents.test_env=true

          # "Int" amount, no cents
          amount = BigDecimal Random.rand(99999)
          card_brand = 'visa'

          # (* optional) used only for CieloPage
          redir_link = "#{@base_url}/api/redirect_receiver"

          # Without cents ,00
          aux = page_transaction_mock(card_brand, amount, redir_link)
          @page_resp = aux[:resp]
          @page_transaction = aux[:transaction]
        end

        # ================    Tests/Expects/Should   ================
        it 'resp should not be null' do
          expect(@page_resp).to_not be_nil
        end

        it 'resp should contain the RequestID' do
          request_id = @page_resp[:request_id]
          expect(request_id).to_not be_nil
          expect(request_id).to be_a Integer
        end

        it 'should setup the rid accessible method' do
          expect(@page_transaction.rid).to_not be_nil
        end

        it 'should setup the purchase_url accessible method' do
          expect(@page_transaction.purchase_url).to_not be_nil
        end

        it 'resp should not have any error' do
          error = @page_resp[:error]
          error = @page_resp['error'] if error.nil?
          expect(error).to be_nil
        end

        it 'resp should be an accessible Operator URL' do
          url = @page_transaction.purchase_url

          expect(url).to_not be_nil
          expect(accessible?(url)).to be_truthy
        end

        it 'must be verifiable' do
          # SetUps
          verify_resp = @page_transaction.verify
          error = verify_resp[:error]
          error = verify_resp['error'] if error.nil?

          # Validations
          expect(error).to be_nil
          expect(verify_resp).to_not be_nil

          # It is just created status
          expect(0..1).to cover(verify_resp[:status][:code])
          expect(verify_resp[:status][:name]).to eq('pending')
        end

        it 'must have a get accessible URL request' do
          url = @page_transaction.url_requested+@page_transaction.request_params.it_keys_to_get_param
          expect(be_accessible(url)).to be_truthy
        end

        it 'amount must be the same as send' do
          expect(@page_transaction.amount.to_i).to be == @page_resp[:transaction][:amount]
        end
      end
    end

    # NOT_OK Operator Page tests
    describe '(BAD REQUEST)' do
      # SetUp/Config
      before(:each) do
        Rents.debug = false
        Rents.test_env = false
        Rents.app_id = 1
        Rents.secret_key = '12312321312$2a$10$NmV9EysKVLe8ItBdl9CHN.LF05bOuDdoOkmfptdbJs7cuaDWksuUu'

        # Not using test_env (so using a app passed not the global app)
        @page_transaction = Rents::Transaction.new({
           test: true,
           card:{brand:'visa'},
           amount: Random.rand(99999), # TODO: no error por rand menor que 99 (* mandatory) Amount in cents (1000 = R$ 10,00)
           redirect_link: "#{@base_url}/api/redirect_receiver" # (* optional) used only for CieloPage
        })

        # Send BuyPage
        @page_resp = @page_transaction.charge_page(full_resp=true)
      end

      it 'resp should contain the RequestID' do
        request_id = JSON.parse(@page_resp).it_keys_to_sym[:request_id]
        expect(request_id).to_not be_nil
        expect(request_id).to be_a Integer
      end

      it 'must have some error' do
        # SetUp vars
        json_resp = JSON.parse(@page_resp)
        error = json_resp[:error]
        error = json_resp['error'] if error.nil?

        # Check what expected
        expect(json_resp).to be_a Hash
        expect(@page_resp.code).to eq(200)
        expect(error).to_not be_nil
      end
    end
  end

  context 'BuyStorePage' do
    # correct Request/Params
    context 'SUCCESS' do
      context 'Permitted request' do
        # OK StorePage tests
        describe 'Transaction charged' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            #  To test it as CAPTURED it mustn't have cents or invalid credit card number!?
            amount = Random.rand(99999).to_s
            amount = "#{amount}00" if amount[amount.length-2, amount.length] != '00'

            # Forcing formatted amount to check if the GEM is sending it with Operator format
            amount.insert amount.length-2, ','

            @store_transaction = Rents::Transaction.new({
              card:{
                brand: 'visa',
                cvv: '123',
                expiration_month: '05',
                expiration_year: '2018',
                number: '4012001037141112',
                holder: Faker::Name.name,
              },

              amount: amount
            })

            # Fake SoldItems added
            @store_transaction.sold_items = fake_sold_items

            # Send StoreCharge
            @resp = @store_transaction.charge_store
          end

          # ================    Tests/Expects/Should   ================
          it 'resp should not be null' do
            expect(@resp).to_not be_nil
          end

          it 'resp should contain the RequestID' do
            request_id = @resp[:request_id]
            expect(request_id).to_not be_nil
            expect(request_id).to be_a Integer
          end

          it 'resp should have a full SUCCESSFUL transaction status' do
            status = @resp[:status]
            expect(status[:name]).to eq('charged')
            expect(status[:code]).to eq(6) # TODO change it to ENUM
            expect(status[:msg].index('Capturada').nil?).to_not be_truthy
          end

          it 'resp should have a full CardObj which is allowed to be stored {:rid, :truncated, :brand}' do
            card = @resp[:card]
            expect(card[:rid]).to be_a Integer
            expect(card[:rid] == 0).to_not be_truthy
            expect(card[:brand]).to eq('visa')
            expect(card[:truncated].index('***')).to_not be_nil
          end

          it 'resp should have the remote reference (TransactionRID)' do
            rid = @resp[:rid]
            expect(rid).to_not be_nil
            expect(rid).to be_a Integer
            expect(rid != 0).to be_truthy
          end
        end

        # TRANSACTION DENIED example
        describe 'Transaction denied' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            #  To test it as DENIED it must have cents or invalid credit card number!?
            amount = Random.rand(99999).to_s
            amount = "#{amount}71" if amount[amount.length-2, amount.length] == '00'

            @store_transaction = Rents::Transaction.new({
              card:{
                brand: 'visa',
                cvv: '321',
                expiration_month: '05',
                expiration_year: '2018',
                number: '9999999999999999',
                holder: Faker::Name.name,
              },
              recurrence_period_id: Rents.enum[:recurrence_periods][:daily],
              amount: amount
            })

            # Fake SoldItems added
            @store_transaction.sold_items = fake_sold_items

            # Send StoreCharge
            @resp = @store_transaction.charge_store
          end

          # ================    Tests/Expects/Should   ================
          it 'resp should not be null' do
            expect(@resp).to_not be_nil
          end

          it 'resp should contain the RequestID' do
            request_id = @resp[:request_id]
            expect(request_id).to_not be_nil
            expect(request_id).to be_a Integer
          end

          it 'resp should have a full SUCCESSFUL transaction status' do
            status = @resp[:status]
            expect(status[:name]).to eq('error')
            expect(status[:code]).to eq(5) # TODO change it to ENUM
            expect(status[:msg].index('não').nil?).to_not be_truthy
          end

          it 'resp should have a full CardObj which is allowed to be stored {:rid, :truncated, :brand}' do
            card = @resp[:card]
            expect(card[:rid]).to be_nil
            expect(card[:brand]).to eq('visa')
            expect(card[:truncated].index('***')).to_not be_nil
          end

          it 'resp should have the remote reference (TransactionRID)' do
            rid = @resp[:rid]
            expect(rid).to_not be_nil
            expect(rid).to be_a Integer
            expect(rid != 0).to be_truthy
          end
        end

        # Create a Subscription (Recurrent charges)
        describe 'Sign a recurrent subscription' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            #  To test it as CAPTURED it mustn't have cents or invalid credit card number!?
            amount = Random.rand(99999).to_s
            amount = "#{amount}00" if amount[amount.length-2, amount.length] != '00'

            @store_transaction = Rents::Transaction.new({
              card:{
                brand: 'visa',
                cvv: '123',
                expiration_month: '05',
                expiration_year: '2018',
                number: '4012001037141112',
                holder: Faker::Name.name
              },
              recurrence_period_id: Rents.enum[:recurrence_periods][:daily],
              amount: amount
            })

            # Fake SoldItems added
            @store_transaction.sold_items = fake_sold_items

            # Send StoreCharge
            @resp = @store_transaction.charge_store
          end

          # ================    Tests/Expects/Should   ================
          it 'resp should not be null' do
            expect(@resp).to_not be_nil
          end

          it 'resp should contain the RequestID' do
            request_id = @resp[:request_id]
            expect(request_id).to_not be_nil
            expect(request_id).to be_a Integer
          end

          it 'resp should have a full SUCCESSFUL transaction status' do
            status = @resp[:status]
            expect(status[:name]).to eq('charged')
            expect(status[:code]).to eq(6) # TODO change it to ENUM
            expect(status[:msg].index('Capturada').nil?).to_not be_truthy
          end

          it 'resp should have a full CardObj which is allowed to be stored {:rid, :truncated, :brand}' do
            card = @resp[:card]
            expect(card[:rid]).to be_a Integer
            expect(card[:rid] == 0).to_not be_truthy
            expect(card[:brand]).to eq('visa')
            expect(card[:truncated].index('***')).to_not be_nil
          end

          it 'resp should have the remote reference (TransactionRID)' do
            rid = @resp[:rid]
            expect(rid).to_not be_nil
            expect(rid).to be_a Integer
            expect(rid != 0).to be_truthy
          end
        end

        # Update a Subscription (Recurrent charges updated)
        describe 'Update the recurrent subscription AMOUNT' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            # Force the amount to be random & to be exact
            amount = Random.rand(99999).to_s
            @amount = "#{amount}00" if amount[amount.length-2, amount.length] != '00'
            @amount = @amount.to_i
            @recurrence_period = Rents.enum[:recurrence_periods][:monthly]

            # Test passing no RID & passing specific RID
            transaction_params = {amount: @amount, recurrence_period: @recurrence_period}
            @empty_transaction = Rents::Transaction.new transaction_params
            transaction_params[:rid] = @global_subscription_rid
            @transaction_with_rid = Rents::Transaction.new transaction_params

            # Send subscription update
            @empty_transaction_resp = @empty_transaction.update_subscription
            @rid_transaction_resp = @transaction_with_rid.update_subscription
          end

          it 'should be Hash' do
            expect(@empty_transaction_resp).to be_a Hash
            expect(@rid_transaction_resp).to be_a Hash
          end

          it 'should be performed the unsubscribe' do
            # SetUpVars
            empty_subscription_resp = @empty_transaction_resp[:subscription]
            resp_amount = empty_subscription_resp[:amount]
            resp_recurrence_period = empty_subscription_resp[:recurrence_period]

            # test those attrs
            expect(resp_amount).to eq(@amount)
            expect(resp_recurrence_period).to eq(@recurrence_period)

            # SetUpVars
            rid_subscription_resp = @rid_transaction_resp[:subscription]
            resp_amount = rid_subscription_resp[:amount]
            resp_recurrence_period = rid_subscription_resp[:recurrence_period]

            # test those attrs
            expect(resp_amount).to eq(@amount)
            expect(resp_recurrence_period).to eq(@recurrence_period)
          end

          it 'should not have any error' do
            expect(@empty_transaction_resp).to_not include :error
            expect(@rid_transaction_resp).to_not include :error
          end
        end

        # Unsubscribe a recurrence payment / signature
        describe 'Unsubscribe a recurrent subscription' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            # Test passing no RID & passing specific RID
            @empty_transaction = Rents::Transaction.new
            @transaction_with_rid = Rents::Transaction.new rid: @global_subscription_rid

            # Send Unsubscribe
            @empty_transaction_resp = @empty_transaction.unsubscribe
            @rid_transaction_resp = @transaction_with_rid.unsubscribe
          end

          it 'should be Hash' do
            expect(@empty_transaction_resp).to be_a Hash
            expect(@rid_transaction_resp).to be_a Hash
          end

          it 'should be performed the unsubscribe' do
            expect(@empty_transaction_resp[:did_unsubscribe]).to be_truthy
            expect(@rid_transaction_resp[:did_unsubscribe]).to be_truthy
          end

          it 'should not have any error' do
            expect(@empty_transaction_resp).to_not include :error
            expect(@rid_transaction_resp).to_not include :error
          end
        end

        # Should return all the bank transaction payments done
        describe 'Subscription payment history' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            # Test passing no RID & passing specific RID
            @transaction = Rents::Transaction.new

            # Send Unsubscribe
            @transaction_resp = @transaction.payment_history
          end

          it 'should be an array' do
            expect(@transaction_resp).to be_a Array
          end

          it 'should not have any error' do
            expect(@transaction_resp.first).to_not include :error
          end

          it 'should have the valid bank_transactions_payment format on it array' do
            transaction = @transaction_resp.first
            valid_keys = [
                :amount, :active, :brand,
                :currency_iso, :status_code, :status_message,
                :next_transaction_at, :created_at, :updated_at
            ]

            valid_keys.each {|key| expect(transaction[key]).to_not be_nil}
          end
        end

        # Should retrieve it only bank transaction done
        describe 'Non-subscription payment history' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            # Test passing no RID & passing specific RID
            global_sample_transaction = get_json("#{@base_url}/api/v1/global_sample_transaction")
            @transaction = Rents::Transaction.new rid: global_sample_transaction[:id]

            # Send Unsubscribe
            @transaction_resp = @transaction.payment_history
          end

          it 'should be a simple hash' do
            expect(@transaction_resp).to be_a Hash
          end

          it 'should not have any error' do
            expect(@transaction_resp).to_not include :error
          end

          it 'should have the valid bank_transactions_payment format' do
            valid_keys = [
                :amount, :active, :brand,
                :currency_iso, :status_code, :status_message,
                :created_at, :updated_at
            ]

            invalid_keys = [
                :next_transaction_at, :recurrence_period_id
            ]

            valid_keys.each {|key| expect(@transaction_resp[key]).to_not be_nil}
            invalid_keys.each {|key| expect(@transaction_resp[key]).to be_nil}
          end
        end

        # Check if a subscription is up-to-date
        describe 'Up-to-date subscription' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            # Test passing no RID & passing specific RID

            @transaction = Rents::Transaction.new

            # Send Unsubscribe
            @transaction_resp = @transaction.up_to_date
          end

          it 'should be Hash' do
            expect(@transaction_resp).to be_a Hash
          end

          it 'should be up-to-date with success message' do
            expect(@transaction_resp[:success]).to_not be_nil
          end

          it 'should not have any error' do
            expect(@transaction_resp).to_not include :error
          end
        end

        # Check if a subscription is up-to-date
        describe 'Subscription PENDING payment, not up-to-date' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=true

            # Test pending subscription payment passing specific RID
            @global_pending_subscription = get_json("#{@base_url}/api/v1/global_pending_subscription")
            @transaction_with_rid = Rents::Transaction.new rid: @global_pending_subscription[:id]

            # Send Unsubscribe
            @rid_transaction_resp = @transaction_with_rid.up_to_date
          end

          it 'should be Hash' do
            expect(@rid_transaction_resp).to be_a Hash
          end

          it 'should not have any error' do
            expect(@rid_transaction_resp).to include :error
          end

          it 'should have the subscription' do
            expect(@rid_transaction_resp[:subscription]).to_not be_nil
          end
        end
      end

      context 'Denied request' do
        # Denying a subscription update
        describe 'Update recurrent subscription DENIED' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=false

            Rents.app_id = 1
            Rents.secret_key = '12312321312$2a$10$NmV9EysKVLe8ItBdl9CHN.LF05bOuDdoOkmfptdbJs7cuaDWksuUu'

            # Force the amount to be random & to be exact
            amount = Random.rand(99999).to_s
            @amount = "#{amount}00" if amount[amount.length-2, amount.length] != '00'
            @amount = @amount.to_i
            @recurrence_period = Rents.enum[:recurrence_periods][:monthly]

            # Test passing no RID & passing specific RID
            transaction_params = {rid: 1, amount: @amount, recurrence_period: @recurrence_period}
            @transaction = Rents::Transaction.new transaction_params

            # Send subscription update
            @subscription_update_resp = @transaction.update_subscription
            @subscription_update_full_resp = @transaction.update_subscription nil, true
          end

          it 'should be Hash' do
            expect(@subscription_update_resp).to be_a Hash
          end

          it 'should not have an error' do
            expect(@subscription_update_resp).to include :error
          end

          it 'should resp with SUCCESS (200)' do
            expect(@subscription_update_full_resp.code).to eq(http_success)
          end
        end

        # Denying an unsubscribe
        describe 'Unsubscribe denied' do
          # ================    SetUp/Config    ================
          before(:all) do
            Rents.debug=true
            Rents.test_env=false

            Rents.app_id = 1
            Rents.secret_key = '12312321312$2a$10$NmV9EysKVLe8ItBdl9CHN.LF05bOuDdoOkmfptdbJs7cuaDWksuUu'

            # Test passing no RID & passing specific RID
            @transaction_with_rid = Rents::Transaction.new rid: 1

            # Send Unsubscribe
            @rid_transaction_resp = @transaction_with_rid.unsubscribe
            @transaction_full_resp = @transaction_with_rid.unsubscribe nil, true
          end

          it 'should be Hash' do
            expect(@rid_transaction_resp).to be_a Hash
          end

          it 'should not have an error' do
            expect(@rid_transaction_resp).to include :error
          end

          it 'should resp with SUCCESS (200)' do
            expect(@transaction_full_resp.code).to eq(http_success)
          end
        end
      end
    end

    # incorrect Request/Params
    context 'BAD REQUEST' do
      # NOT_OK on the authentication Store Page tests
      describe 'Authentication failing' do
        # SetUp/Config
        before(:all) do
          Rents.debug = true
          Rents.test_env = false
          Rents.app_id = 1
          Rents.secret_key = '12312321312$2a$10$NmV9EysKVLe8ItBdl9CHN.LF05bOuDdoOkmfptdbJs7cuaDWksuUu'

          amount = Random.rand(99999).to_s
          amount = "#{amount}00" if amount[amount.length-2, amount.length] != '00'

          @store_transaction = Rents::Transaction.new({
              card:{
                brand: 'visa',
                cvv: '321',
                expiration_month: '05',
                expiration_year: '2018',
                number: '9999999999999999',
                holder: Faker::Name.name,
              },

              amount: amount
          })

          # Fake SoldItems added
          @store_transaction.sold_items = fake_sold_items

          # Send StoreCharge
          @resp = @store_transaction.charge_store
        end

        # ================    Tests/Expects/Should   ================
        it 'resp should exist (do not be empty)' do
          expect(@resp).to_not be_nil
        end

        it 'resp should contain the RequestID' do
          request_id = @resp[:request_id]
          expect(request_id).to_not be_nil
          expect(request_id).to be_a Integer
        end

        it 'resp should contain auth error' do
          error = @resp[:error].downcase
          expect(error).to_not be_nil
          expect(error.index('auth error').nil?).to_not be_truthy
          expect(error.index('AppID'.downcase).nil?).to_not be_truthy
          expect(error.index('SecretKey'.downcase).nil?).to_not be_truthy
        end
      end
    end
  end

  context 'Proxy' do
    it 'should validate the proxy' do
      uncomment_proxy_yml
      expect(Rents.proxy).to_not be_nil
      comment_proxy_yml
    end
  end

  context 'Remote sample connection test' do
    before(:all) do
      Rents.debug=false
      Rents.test_env=false
      @remote = Rents::Status.new
    end

    it 'must have a response JSON' do
      expect(@remote.response).to be_a Hash
    end

    it 'must have a message' do
      expect(@remote.message).to be_a String
    end

    it 'must have an API code' do
      expect(@remote.api_code).to be_a Integer
    end
  end
end